function jaro_similarity(s1, s2)
{
    s1 = s1.normalize('NFC')
    s2 = s2.normalize('NFC')

    let len1 = s1.length
    let len2 = s2.length

    let match_bound = Math.floor (Math.max(len1, len2) /2) -1

    let matches = 0
    let transpositions = 0

    let flagged_1 = []
    let flagged_2 = []

    for (let i = 0; i < len1; i++){
        i = parseInt(i)
        let upperbound = Math.min( (i + match_bound +1), len2)
        let lowerbound = Math.max(0, i - match_bound)
        for(let j = lowerbound; j < upperbound; j++ ) {

            if (s1[i] === s2[j] && !(flagged_2.includes(j))) {

                matches++
                flagged_1.push(i)
                flagged_2.push(j)
                break
            }
        }
    }
    flagged_2.sort(function (a, b){return a-b})
    for (let i = 0; i<flagged_1.length;i++){
        if (s1[flagged_1[i]] !== s2[flagged_2[i]]) {
            transpositions++
        }
    }

    if (matches === 0) {
        return 0;
    } else {
        return (1/3) * (matches/len1 +matches/len2 + (matches - Math.floor(transpositions/2))/matches);
    }
}

function jaro_winkler_similarity(s1, s2, p=0.1, max_l=4) {
    let jaro_sim = jaro_similarity(s1, s2)
    let l = 0
    for (let i = 0; i<s1.length;i++) {
        if (s1[i] === s2[i]) {
            l++
        } else {break}
        if (l===max_l) {break}
    }
    return jaro_sim + (l*p*(1-jaro_sim))
}

function demo() {
    let string_distance_examples1 = [
        ("rain"),
        ("abcdef"),
        ("language"),
        ("language"),
        ("language"),
    ]
    let string_distance_examples2 = [
        ("shine"),
        ("abcdef"),
        ("lnaguaeg"),
        ("lnaugage"),
        ("lngauage"),
    ]
    for (let i = 0; i < string_distance_examples1.length; i++) {
        console.log(string_distance_examples1[i], string_distance_examples2[i], jaro_similarity(string_distance_examples1[i], string_distance_examples2[i]), jaro_winkler_similarity(string_distance_examples1[i], string_distance_examples2[i]))
    }
}

demo()

//console.log(jaro_winkler_similarity("Intelligenz der Computer und Matematik", "Inteligens"))
