CREATE OR REPLACE FUNCTION array_sort (arr int[])
RETURNS int[]
AS $$
  SELECT ARRAY(SELECT UNNEST(arr) ORDER BY 1)
$$ LANGUAGE SQL IMMUTABLE;
CREATE OR REPLACE FUNCTION fn_jaro_similarity(s1 varchar, s2 varchar)
RETURNS numeric AS
$body$
DECLARE
	--length of each string
	len1 numeric;
	len2 numeric;
	--upperbound of the distance for being a matched character
	match_bound int;
	--init for matches and transpositions
	matches numeric := 0;
	transpositions numeric := 0;
	--positions of characters that are matched with a character in the other string
	flagged_1 int[];
	flagged_2 int[];
	--upperbound and lowerbound (calculated with match_bound)
	upperbound int :=0;
	lowerbound int :=0;
	--counter 
	x int := 0;
BEGIN
	len1 := LENGTH(s1);
	len2 := LENGTH(s2);
	match_bound := FLOOR(GREATEST(len1, len2)/2)-1;
	flagged_1 := ARRAY[]::INT[];
	flagged_2 := ARRAY[]::INT[];
	FOR i in 1..LENGTH(s1) BY 1 --check each letter by s1
	LOOP
		upperbound := LEAST(i + match_bound, len2);
		lowerbound := GREATEST(1, i - match_bound);
		FOR j IN lowerbound..upperbound BY 1 --matching the letters in s2 with the letter from s1
		LOOP
			IF (SUBSTRING(s1,i,1)=SUBSTRING(s2,j,1) AND j != ALL(flagged_2)) THEN
				matches := matches+1;
				flagged_1 = array_append(flagged_1, i);
				flagged_2 = array_append(flagged_2, j);

				EXIT;
			END IF;
		END LOOP;
	END LOOP;
	flagged_2 := array_sort(flagged_2);
	FOREACH x IN ARRAY flagged_1
	LOOP
		IF (SUBSTRING(s1,flagged_1[x],1)!=SUBSTRING(s2,flagged_2[x],1)) THEN
			transpositions := transpositions + 1;
		END IF;
	END LOOP;
	IF (matches = 0) THEN
		RETURN 0;
	ELSE 
		RETURN (matches/len1 + matches/len2 + ((matches - FLOOR(transpositions/2))/matches))/3;
	END IF;
END;
$body$
LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION fn_jaro_winkler_similarity(s1 varchar, s2 varchar)
RETURNS numeric AS
$body$
DECLARE
	jaro_sim numeric := fn_jaro_similarity(s1, s2);	-- jaro similarity of s1 and s2
	l numeric := 0;									-- length of common prefix at the start of the strings
	scaling_factor numeric := 0.1;					-- constant scaling factor (will fall within [0, 1] bound) default is .1
	max_l int := 4;									-- upperbound for common prefix 
BEGIN
	FOR i IN 1..LENGTH(s1) BY 1
	LOOP
		IF (SUBSTRING(s1,i,1) = substring(s2,i,1)) THEN
			l := l + 1;
		ELSE
			EXIT;
		END IF;
		IF (l = max_l) THEN
			EXIT;
		END IF;
	END LOOP;
	RETURN (jaro_sim + (l * scaling_factor * (1 - jaro_sim)));
END;
$body$
LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION fn_demo()
RETURNS TABLE (s1 varchar, s2 varchar, similarity numeric(1, 5)) AS
$body$
DECLARE 
	rec RECORD;
BEGIN
	FOR rec IN (SELECT string_1, string_2 FROM comparison_strings)
	LOOP
		SELECT rec.string_1, rec.string_2, fn_jaro_winkler_similarity(rec.string_1, rec.string_2)
		INTO s1, s2, similarity;
		RAISE NOTICE 'jaro_winkler_similarity of % and % is %', rec.string_1, rec.string_2, similarity;
		RETURN NEXT;
	END LOOP;
END
$body$
LANGUAGE plpgsql;

SELECT fn_demo()
DROP FUNCTION fn_demo()



